﻿//!CompilerOption:Optimize:On
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;
using System.Windows.Media;
using System.Xml.Linq;
using CommonBehaviors.Actions;
using katzerle.Composites;
using Levelbot.Actions.Death;
using Levelbot.Decorators.Death;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Profiles;
using Styx.CommonBot.Routines;
using Styx.Helpers;

using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.TreeSharp;
using Action = Styx.TreeSharp.Action;
using Color = System.Windows.Media.Color;

namespace katzerle
{
    public enum PathingType
    {
        Circle,
        Bounce
    }

    public class FlyAroundBot : BotBase
    {
       public static List<WoWPoint> WayPoints = new List<WoWPoint>();
        private static int _currentIndex;
        public readonly Version Version = new Version(1,0);
        private readonly LocalPlayer _me = StyxWoW.Me;

        public FlyAroundBot()
        {
            BotEvents.Profile.OnNewOuterProfileLoaded += Profile_OnNewOuterProfileLoaded;
        }

        public static WoWPoint CurrentPoint
        {
            get
            {
                return WayPoints != null && WayPoints.Count > 0
                           ? WayPoints[_currentIndex]
                           : WoWPoint.Zero;
            }
        }

        #region overrides

        private PrioritySelector _root;

        public override string Name
        {
            get { return "Fly Around Bot"; }
        }

        public override PulseFlags PulseFlags
        {
            get { return PulseFlags.All; }
        }

        public override void Pulse()
        {
        }

        private DateTime _pulseTimestamp;
        public override Composite Root
        {
            get
            {
                return _root ?? (_root = new PrioritySelector(
                    // Is bot dead? if so, release and run back to corpse
                                             new Decorator(c => !_me.IsAlive,
                                                           new PrioritySelector(
                                                               new DecoratorNeedToRelease(new ActionReleaseFromCorpse()),
                                                               new DecoratorNeedToMoveToCorpse(new ActionMoveToCorpse()),
                                                               new DecoratorNeedToTakeCorpse(new ActionRetrieveCorpse()),
                                                               new ActionSuceedIfDeadOrGhost()
                                                               )),
                    // If bot is in combat call the CC routine
                                             new Decorator(c => _me.Combat && !_me.IsFlying,
                                                 new PrioritySelector(
                                                            // Use the Behavior
                                                            new Decorator(ctx => RoutineManager.Current.CombatBehavior != null,
                                                                new PrioritySelector(
                                                                    RoutineManager.Current.CombatBehavior,
                                                                    new Action(delegate { return RunStatus.Success; })
                                                                    )),

                                                            // Don't use the Behavior
                                                            new Sequence(
                                                                new Action(ret => TreeRoot.StatusText = "Combat"),
                                                                new Action(ret => RoutineManager.Current.Combat()))

                                                            )),
                    // If bot needs to rest then call the CC rest behavior
                                             new Decorator(
                                                 c =>
                                                 RoutineManager.Current.NeedRest && !StyxWoW.Me.IsCasting &&
                                                 !_me.IsFlying && !StyxWoW.Me.Combat,
                                                 new PrioritySelector(
                                                            // Use the Behavior
                                                            new Decorator(ctx => RoutineManager.Current.RestBehavior != null,
                                                                new PrioritySelector(
                                                                    RoutineManager.Current.RestBehavior,
                                                                    new Action(delegate { return RunStatus.Success; })
                                                                    )),

                                                            // Don't use the Behavior
                                                            new Sequence(
                                                                new Action(ret => TreeRoot.StatusText = "Rest"),
                                                                new Action(ret => RoutineManager.Current.Rest()))
                                                     )),
                    // mail and repair if bags are full or items have low durability. logout if profile doesn't have mailbox and vendor.
                                             new Decorator(c => _me.BagsFull || _me.DurabilityPercent <= 0.2 &&
                                                                (BotPoi.Current == null ||
                                                                 BotPoi.Current.Type != PoiType.Mail &&
                                                                 BotPoi.Current.Type != PoiType.Repair &&
                                                                 BotPoi.Current.Type != PoiType.InnKeeper),
                                                           new Action(c =>
                                                                          {
                                                                              if (ProfileManager.CurrentOuterProfile !=
                                                                                  null &&
                                                                                  ProfileManager.CurrentOuterProfile.
                                                                                      MailboxManager != null)
                                                                              {
                                                                                  Mailbox mbox =
                                                                                      ProfileManager.CurrentOuterProfile
                                                                                          .MailboxManager.
                                                                                          GetClosestMailbox();
                                                                                  if (mbox != null && !String.IsNullOrEmpty(CharacterSettings.Instance.MailRecipient))
                                                                                  {
                                                                                      BotPoi.Current = new BotPoi(mbox);
                                                                                  }
                                                                                  else
                                                                                  {
                                                                                      Vendor ven = ProfileManager.CurrentOuterProfile.VendorManager.GetClosestVendor();
                                                                                      if (ven != null)
                                                                                          BotPoi.Current = new BotPoi(ven, PoiType.Repair);
                                                                                      else
                                                                                          // we'll use this POI to hearth+Logout...
                                                                                          BotPoi.Current = new BotPoi(PoiType.InnKeeper);
                                                                                  }
                                                                              }
                                                                              return RunStatus.Failure;
                                                                          })),
                                             new Decorator(
                                                 c => BotPoi.Current != null && BotPoi.Current.Type == PoiType.Mail,
                                                 new MailAction()),
                                             new Decorator(
                                                 c => BotPoi.Current != null && BotPoi.Current.Type == PoiType.Repair,
                                                 new VendorAction()),
                                             new Decorator(
                                                 c => BotPoi.Current != null && BotPoi.Current.Type == PoiType.InnKeeper,
                                                 new LogoutAction()),
                    // loot Any dead lootable NPCs if setting is enabled.
                                             new Decorator(c => true,
                                                           new LootNPCsAction()),
                    // follow the path...
                                             new FollowPathAction()
                                             ));
            }
        }
        #endregion


        #region Profile

        private void Profile_OnNewOuterProfileLoaded(BotEvents.Profile.NewProfileLoadedEventArgs args)
        {
            try
            {
                LoadWayPoints(ProfileManager.CurrentProfile);
            }
            catch (Exception ex)
            {
                Logging.WriteException(ex);
            }
        }

        private void LoadWayPoints(Profile profile)
        {
            WayPoints.Clear();
            if (profile != null && profile.GrindArea != null)
            {
                if (profile.GrindArea.Hotspots != null)
                {
                    WayPoints = profile.GrindArea.Hotspots.ConvertAll(hs => hs.Position);
                    WoWPoint closestPoint =
                        WayPoints.OrderBy(u => u.Distance(StyxWoW.Me.Location)).FirstOrDefault();
                    _currentIndex = WayPoints.FindIndex(w => w == closestPoint);
                }
                else
                    WayPoints = new List<WoWPoint>();
            }
        }

        public static void CycleToNextPoint()
        {
            if (WayPoints != null)
            {
                if (_currentIndex >= WayPoints.Count - 1)
                {
                    _currentIndex = 0;
                }
                else
                    _currentIndex++;
            }
        }

        private static WoWPoint GetNextWayPoint()
        {
            int i = _currentIndex + 1;
            if (i >= WayPoints.Count)
            {
                i = 0;
            }
            if (WayPoints != null && i < WayPoints.Count)
                return WayPoints[i];
            return WoWPoint.Zero;
        }
        #endregion


        public static void Log(string format, params object[] args)
        {
            Logging.Write(Colors.LightBlue, String.Format("FlyAround: {0}", format), args);
        }

        public void Err(string format, params object[] args)
        {
            Logging.Write(Colors.Red, String.Format("FlyAround: {0}", format), args);
        }

        public void Debug(string format, params object[] args)
        {
            Logging.WriteDiagnostic(Colors.LightBlue, String.Format("FlyAround: {0}", format), args);
        }
    }
}